using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace StyledControls
{

	public class StyledToolBar : ToolBar
	{
		public StyledToolBar() 
		{
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);
			Divider = false;
			DropDownArrows = false;
			ButtonSize = new Size(28, 24);
		}

		protected override void OnResize(EventArgs e)
		{
			RebuildDrawData();

			base.OnResize(e);
		}
		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged(e);

			lightColor = ControlPaint.LightLight(BackColor);
			lightLightColor = ControlPaint.LightLight(lightColor);
			darkColor = BackColor;
		}

		protected override void OnMouseLeave(EventArgs e)
		{
			if(hover != -1)
			{
				Invalidate(Buttons[hover].Rectangle);
				hover = -1;
			}

			base.OnMouseLeave(e);
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			ToolBarButton b = GetButtonAtPoint(e.X, e.Y);
			if(b != null)
			{
				int ind = Buttons.IndexOf(b);
				if(hover != ind)
				{
					if(hover != -1)
						Invalidate(Buttons[hover].Rectangle);

					hover = ind;
					Invalidate(b.Rectangle);
				}
			}
			if(b == null && hover != -1)
			{
				Invalidate(Buttons[hover].Rectangle);
				hover = -1;
			}

			base.OnMouseMove(e);
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Left)
			{
				ToolBarButton b = GetButtonAtPoint(e.X, e.Y);
				if(b != null)
				{
					pushed = Buttons.IndexOf(b);
					Invalidate(b.Rectangle);
				}
			}

			base.OnMouseMove(e);
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Left)// && Capture)
			{
				if(pushed != -1)
				{
					int p = pushed;
					pushed = -1;
					Invalidate(Buttons[p].Rectangle);
				}
//				ToolBarButton b = GetButtonAtPoint(e.X, e.Y);
//				if(b != null)
//				{
//					pushed = -1;
//					Invalidate(b.Rectangle);
//				}
			}

			base.OnMouseMove(e);
		}

		private ToolBarButton GetButtonAtPoint(int x, int y)
		{
			foreach(ToolBarButton b in Buttons)
				if(b.Rectangle.Contains(x, y))
					return b;

			return null;
		}

		protected GraphicsPath startButtonPath;
		protected GraphicsPath midButtonPath;
		protected GraphicsPath endButtonPath;
		protected GraphicsPath startGlowPath;
		protected GraphicsPath midGlowPath;
		protected GraphicsPath endGlowPath;
		protected Rectangle hRect;
		protected Rectangle buttonRect;
		protected Rectangle glowRect;
		protected Color lightColor;
		protected Color lightLightColor;
		protected Color darkColor;
		protected Color pFrontColor;
		protected Color pBackColor;
		protected int hover = -1;
		protected int pushed = -1;
		protected int focused = -1;

		protected virtual void RebuildDrawData()
		{
			int w = ButtonSize.Width;
			int h = ButtonSize.Height;

			int hh = h / 2;
			int wlen = w - hh;
			int hlen = h - hh;
			int gh = (int)(h / 1.6f);

			buttonRect = new Rectangle(0, 0, w, h);
			glowRect = new Rectangle(0, -1, w, gh + 2);
			hRect = new Rectangle(0, 0, Width, Height / 2);

			//STARTBUTTON

			startButtonPath = new GraphicsPath();
			startButtonPath.AddArc(0, hlen, hh, hh, 90, 90);
			startButtonPath.AddArc(0, 0, hh, hh, 180, 90);
			startButtonPath.AddLine(w, 0, w, h);
			startButtonPath.CloseFigure();

			startGlowPath = new GraphicsPath();
			startGlowPath.AddArc(0, 0, gh, gh, 90, 180);
			startGlowPath.AddLine(w, 0, w, gh);
			startGlowPath.CloseFigure();

			//MIDBUTTON

			midButtonPath = new GraphicsPath();
			midButtonPath.AddRectangle(new Rectangle(0, 0, w, h));

			midGlowPath = new GraphicsPath();
			midGlowPath.AddRectangle(new Rectangle(0, 0, w, gh));
			
			//ENDBUTTON

			endButtonPath = new GraphicsPath();
			endButtonPath.AddLine(0, h, 0, 0);
			endButtonPath.AddArc(wlen, 0, hh, hh, -90, 90);
			endButtonPath.AddArc(wlen, hlen, hh, hh, 0, 90);
			endButtonPath.CloseFigure();

			endGlowPath = new GraphicsPath();
			endGlowPath.AddLine(0, gh, 0, 0);
			endGlowPath.AddArc(w - gh, 0, gh, gh, -90, 180);
			endGlowPath.CloseFigure();
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.HighQuality;
			
			using(LinearGradientBrush gb = new LinearGradientBrush(hRect, darkColor, lightColor, LinearGradientMode.Vertical))
			{
				gb.WrapMode = WrapMode.TileFlipXY;
				g.FillRectangle(gb, ClientRectangle);
			}
			
			for(int i = 0; i < Buttons.Count; i++)
			{
				if(Buttons[i].Style != ToolBarButtonStyle.Separator)
				{
					if(i == 0 || Buttons[i - 1].Style == ToolBarButtonStyle.Separator)
						DrawButton(g, Buttons[i].Rectangle, i, StyledToolBarButtonStyle.Start);
					else if(i == Buttons.Count - 1 || Buttons[i + 1].Style == ToolBarButtonStyle.Separator)
						DrawButton(g, Buttons[i].Rectangle, i, StyledToolBarButtonStyle.End);
					else
						DrawButton(g, Buttons[i].Rectangle, i, StyledToolBarButtonStyle.Mid);

					if(ImageList != null && ImageList.Images.Count > 0 && Buttons[i].ImageIndex >= 0 && ImageList.Images[Buttons[i].ImageIndex] != null)
						DrawImage(g, Buttons[i].Rectangle, Buttons[i].ImageIndex);
				}
			}

////////////////////////////////////////////////////
//			Console.WriteLine("------------------");
		}

		protected virtual void DrawImage(Graphics g, Rectangle rect, int index)
		{
			Image img = ImageList.Images[index];
			int x = rect.X + (rect.Width - ImageSize.Width) / 2;
			int y = rect.Y + (rect.Height - ImageSize.Height) / 2;
			g.DrawImageUnscaled(ImageList.Images[index], x, y);
		}

		protected virtual void DrawButton(Graphics g, Rectangle rect, int index, StyledToolBarButtonStyle style)
		{
			GraphicsPath bp, gp;
			switch(style)
			{
				case (StyledToolBarButtonStyle.Start) :
				{
					bp = (GraphicsPath)startButtonPath.Clone();
					gp = (GraphicsPath)startGlowPath.Clone();
					break;
				}
				case (StyledToolBarButtonStyle.End) :
				{
					bp = (GraphicsPath)endButtonPath.Clone();
					gp = (GraphicsPath)endGlowPath.Clone();
					break;
				}
				default :
				{
					bp = (GraphicsPath)midButtonPath.Clone();
					gp = (GraphicsPath)midGlowPath.Clone();
					break;
				}
			}

			Matrix m = new Matrix();
			m.Translate(rect.X, 0);
			bp.Transform(m);
			gp.Transform(m);

			Rectangle gr = glowRect;
			gr.Offset(rect.X, 0);

			using(SolidBrush sb = new SolidBrush(darkColor))
				g.FillPath(sb, bp);

			Color gcol = hover == index ? lightLightColor : lightColor;
			using(LinearGradientBrush gb = new LinearGradientBrush(gr, gcol, darkColor, LinearGradientMode.Vertical))
				g.FillPath(gb, gp);

			if(pushed == index)
				using(LinearGradientBrush gb = new LinearGradientBrush(rect, lightLightColor, darkColor, LinearGradientMode.Vertical))
				using(Pen intPen = new Pen(gb, 4))
					g.DrawPath(intPen, bp);
			else
				using(Pen intPen = new Pen(lightColor, 2))
					g.DrawPath(intPen, bp);



///////////////////////////////////////////////
//			Console.WriteLine(bp.PathPoints[0]);
	
//			m.Translate(-rect.X, 0);
//			bp.Transform(m);
//			gp.Transform(m);
		}

	}

	public enum StyledToolBarButtonStyle
	{
		Start,
		Mid,
		End
	}
}
